"""
This is ropehints module which configures Snaked's python type hinting system.

Enable Django support::

    from snaked.plugins.python.djangohints import add_django_support
    add_django_support(provider)

Enable PyGtk support::

    from snaked.plugins.python.pygtkhints import add_gtk_support
    pygtk = add_gtk_support(provider)

For detailed description look at http://packages.python.org/snaked/python.html#type-hints
"""

def init(provider):
    pass